function [dg,dgDV,dgMD,dgSM,dgFA,dgDM,ue,un,uv]=pCDMgravity(X,Y,X0,Y0,...
    depth,omegaX,omegaY,omegaZ,DVx,DVy,DVz,rho,DM,gamma,nu)
% pCDMgravity
% calculates surface gravity changes and surface displacements caused by a 
% point CDM in a uniform elastic half-space.
% 
% pCDM: point Compound Dislocation Model
% PTD: Point Tensile Dislocation
% EFCS: Earth-Fixed Coordinate System
%
% INPUTS
% X and Y [m]:
% Horizontal coordinates of calculation points in EFCS (East, North, Up). 
% X and Y must have the same number of elements.
%
% X0, Y0 and depth [m]:
% Horizontal coordinates and depth of the point CDM. The depth must be a 
% positive value.
%
% omegaX, omegaY and omegaZ [degree]:
% Clockwise rotation angles about the X, Y and Z axes, respectively, that 
% specify the orientation of the point CDM in space.
%
% DVx, DVy and DVz [m^3]:
% Potencies of the PTDs.
% 
% rho [kg/m^3]:
% The host rock density.
% 
% DM [kg]:
% The net mass of intruded fluids.
% 
% gamma [1/s^2]:
% The free-air gradient. gamma must be a negative number. 
% The theoretical value for gamma is approximately -0.3086e-5 1/s^2.
% 
% nu:
% Poisson's ratio.
% 
% 
% OUTPUTS
% dg [m/s^2]:
% Total surface gravity change.
% 
% dgDV [m/s^2]:
% Surface gravity changes due to interface cavity (potency DV).
% 
% dgMD [m/s^2]: 
% Surface gravity changes due to medium dilatation (in both host rock and 
% source region!).
% 
% dgSM [m/s^2]: 
% Surface gravity changes due to displaced surface mass (Bouguer plate).
% 
% dgFA [m/s^2]:
% Surface gravity changes due to vertical displacement of gravimeter (free 
% air effect).
% 
% dgDM [m/s^2]:
% Surface gravity changes due to intruded fluid mass.
% 
% Note: 1 microGal = 1e-8 m/s^2!
% 
% ue, un and uv [m]:
% Surface displacement components in EFCS.
%
%
% Example-1: Compare gravity change contributions for massless sources of 
% various source aspect ratio:
% 
% [x,y] = meshgrid(0:10:5000,0);
% x0 = 0; y0 = 0; depth = 1000; omegaX = 0; omegaY = 0; omegaZ = 0;
% rho = 2600; DM = 0; gamma = -0.3086e-5; nu = 0.25;
% DVx1 = 1e6; DVy1 = 1e6; DVz1 = 1e6; % sphere
% DVx2 = 1e6; DVy2 = 1e6; DVz2 = 0.5e6; % prolate
% [dg1,dgDV1,dgMD1,dgSM1,dgFA1]=pCDMgravity(x,y,x0,y0,depth,...
%     omegaX,omegaY,omegaZ,DVx1,DVy1,DVz1,rho,DM,gamma,nu);
% [dg2,dgDV2,dgMD2,dgSM2,dgFA2]=pCDMgravity(x,y,x0,y0,depth,...
%     omegaX,omegaY,omegaZ,DVx2,DVy2,DVz2,rho,DM,gamma,nu);
% figure
% subplot(121)
% plot(x/depth,dg1/max(dgSM1),'r','LineWidth',2)
% hold on
% title('Spherical')
% xlabel('x/depth')
% ylabel('Gravity changes / max(\Delta{g}_{SM})')
% plot(x/depth,dgDV1/max(dgSM1),'g')
% plot(x/depth,dgMD1/max(dgSM1),'b')
% plot(x/depth,dgSM1/max(dgSM1),'k')
% plot(x/depth,dgFA1/max(dgSM1),'k--','LineWidth',2)
% plot(x/depth,(dgDV1+dgMD1+dgSM1)/max(dgSM1),'m')
% legend('\delta{g}','\Delta{g}_{\Delta{V}}','\Delta{g}_{MD}',...
%     '\Delta{g}_{SM}','\Delta{g}_{FA}',...
%     '\Delta{g}_{\Delta{V}}+\Delta{g}_{MD}+\Delta{g}_{SM}',...
%     'Location','southeast')
% subplot(122)
% plot(x/depth,dg2/max(dgSM2),'r','LineWidth',2)
% hold on
% title('Prolate')
% xlabel('x/depth')
% ylabel('Gravity changes / max(\Delta{g}_{SM})')
% plot(x/depth,dgDV2/max(dgSM2),'g')
% plot(x/depth,dgMD2/max(dgSM2),'b')
% plot(x/depth,dgSM2/max(dgSM2),'k')
% plot(x/depth,dgFA2/max(dgSM2),'k--','LineWidth',2)
% plot(x/depth,(dgDV2+dgMD2+dgSM2)/max(dgSM2),'m')
% legend('\delta{g}','\Delta{g}_{\Delta{V}}','\Delta{g}_{MD}',...
%     '\Delta{g}_{SM}','\Delta{g}_{FA}',...
%     '\Delta{g}_{\Delta{V}}+\Delta{g}_{MD}+\Delta{g}_{SM}',...
%     'Location','southeast')
% set(gcf,'Position',get(0,'ScreenSize'))
% 
% Example-2: Calculate gravity changes, radial and vertical displacements; 
% see Figure 4 in Nikkhoo and Rivalta (2022)
%
% [x,y] = meshgrid(0:10:4000,0);
% x0 = 0; y0 = 0; omegaX = 0; omegaY = 0; omegaZ = 0; rho = 2600; DM = 0;
% gamma = -0.3086e-5; nu = 0.25;  
% DVx1 = 1e6; DVy1 = 1e6; DVz1 = 1e6; % sphere
% DVx2 = 0; DVy2 = 0; DVz2 = 1e6; % sill
% DVx3 = 1e6; DVy3 = 1e6; DVz3 = 0.44e6; % prolate
% 
% %%% Similar vertical displacements
% depth1 = 1e3; depth2 = 1.44e3; depth3 = 0.55e3;
% [dg1,~,~,dgSM1,dgFA1,~,ue1,~,uv1]=pCDMgravity(x,y,x0,y0,depth1,...
%     omegaX,omegaY,omegaZ,DVx1,DVy1,DVz1,rho,DM,gamma,nu);
% [dg2,~,~,dgSM2,dgFA2,~,ue2,~,uv2]=pCDMgravity(x,y,x0,y0,depth2,...
%     omegaX,omegaY,omegaZ,DVx2,DVy2,DVz2,rho,DM,gamma,nu);
% [dg3,~,~,dgSM3,dgFA3,~,ue3,~,uv3]=pCDMgravity(x,y,x0,y0,depth3,...
%     omegaX,omegaY,omegaZ,DVx3,DVy3,DVz3,rho,DM,gamma,nu);
% figure
% subplot(222)
% plot(x/depth1,(dg1-dgFA1)./max(dgSM1),'b')
% title('Gravity changes')
% xlabel('Distance (r/depth)')
% ylabel('Normalized Gravity changes')
% hold on
% plot(x/depth1,(dg2-dgFA2)./max(dgSM2),'r')
% plot(x/depth1,(dg3-dgFA3)./max(dgSM3),'k--')
% set(gca,'YLim',[-1.5 1])
% legend('Mogi','Sill','Prolate','Location','southeast')
% subplot(223)
% plot(x/depth1,uv1./max(uv1),'b')
% title('Vertical displacements')
% xlabel('Distance (r/depth)')
% ylabel('Normalized displacements')
% hold on
% plot(x/depth1,uv2./max(uv2),'r')
% plot(x/depth1,uv3./max(uv3),'k--')
% set(gca,'YLim',[0 1.2])
% subplot(224)
% plot(x/depth1,ue1./max(uv1),'b')
% title('Radial displacements')
% xlabel('Distance (r/depth)')
% ylabel('Normalized displacements')
% hold on
% plot(x/depth1,ue2./max(uv2),'r')
% plot(x/depth1,ue3./max(uv3),'k--')
% set(gca,'YLim',[0 1.2])
% ScSz = get(0,'ScreenSize');
% set(gcf,'Position',[1 1 floor(ScSz(3)/2)-10 floor(ScSz(4))])
% 
% %%% Similar radial displacements 
% depth1 = 1e3; depth2 = 1.75e3; depth3 = 0.75e3;
% [dg1,~,~,dgSM1,dgFA1,~,ue1,~,uv1]=pCDMgravity(x,y,x0,y0,depth1,...
%     omegaX,omegaY,omegaZ,DVx1,DVy1,DVz1,rho,DM,gamma,nu);
% [dg2,~,~,dgSM2,dgFA2,~,ue2,~,uv2]=pCDMgravity(x,y,x0,y0,depth2,...
%     omegaX,omegaY,omegaZ,DVx2,DVy2,DVz2,rho,DM,gamma,nu);
% [dg3,~,~,dgSM3,dgFA3,~,ue3,~,uv3]=pCDMgravity(x,y,x0,y0,depth3,...
%     omegaX,omegaY,omegaZ,DVx3,DVy3,DVz3,rho,DM,gamma,nu);
% figure
% subplot(222)
% plot(x/depth1,(dg1-dgFA1)./max(dgSM1),'y')
% title('Gravity changes')
% xlabel('Distance (r/depth)')
% ylabel('Normalized Gravity changes')
% hold on
% plot(x/depth1,(dg2-dgFA2)./max(dgSM2),'r')
% plot(x/depth1,(dg3-dgFA3)./max(dgSM3),'k--')
% set(gca,'YLim',[-1.5 1])
% legend('Mogi','Sill','Prolate','Location','southeast')
% subplot(223)
% plot(x/depth1,uv1./max(ue1),'y')
% title('Vertical displacements')
% xlabel('Distance (r/depth)')
% ylabel('Normalized displacements')
% hold on
% plot(x/depth1,uv2./max(ue2),'r')
% plot(x/depth1,uv3./max(ue3),'k--')
% set(gca,'YLim',[0 4])
% subplot(224)
% plot(x/depth1,ue1./max(ue1),'y')
% title('Radial displacements')
% xlabel('Distance (r/depth)')
% ylabel('Normalized displacements')
% hold on
% plot(x/depth1,ue2./max(ue2),'r')
% plot(x/depth1,ue3./max(ue3),'k--')
% set(gca,'YLim',[0 4])
% set(gcf,'Position',[10+ceil(ScSz(3)/2) 1 floor(ScSz(3)/2)-10 ScSz(4)])

% Reference journal articles:
% 1)
% Nikkhoo, M., Rivalta, E. (2022):
% Analytical solutions for gravity changes caused by triaxial volumetric 
% sources. Geophysical Research Letters, doi: 10.1029/2021GL095442
% 
% 2)
% Nikkhoo, M., Walter, T. R., Lundgren, P. R., Prats-Iraola, P. (2017):
% Compound dislocation models (CDMs) for volcano deformation analyses.
% Geophysical Journal International, doi: 10.1093/gji/ggw427
% 
% 3)
% Okubo, S. (1991):
% Potential and gravity changes raised by point dislocations. Geophysical 
% journal international, doi: 10.1111/j.1365-246X.1991.tb00797.x

% Copyright (c) 2022 Mehdi Nikkhoo
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files
% (the "Software"), to deal in the Software without restriction, including
% without limitation the rights to use, copy, modify, merge, publish,
% distribute, sublicense, and/or sell copies of the Software, and to permit
% persons to whom the Software is furnished to do so, subject to the
% following conditions:
%
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
% NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
% OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
% USE OR OTHER DEALINGS IN THE SOFTWARE.

% I appreciate any comments or bug reports.

% Mehdi Nikkhoo
% Created: 2020 Sep 12
% Last modified: 2022 Oct 24
%
% Section 2.1, Physics of Earthquakes and Volcanoes
% Department 2, Geophysics
% Helmholtz Centre Potsdam
% German Research Centre for Geosciences (GFZ)
%
% email:
% mehdi.nikkhoo@gfz-potsdam.de
% mehdi.nikkhoo@gmail.com
%
% website:
% http://www.volcanodeformation.com


if gamma>0
    error('Input error: gamma must be a negetive number!')
end

if depth<=0
    error('Input error: depth must be a positive number!')
end

% The potencies must have the same sign
DVsign = [sign(DVx) sign(DVy) sign(DVz)];
if any(DVsign>0) && any(DVsign<0)
    error('Input error: DVx, DVy and DVz must have the same sign!')
end

G = 6.67408e-11; % Universal gravitational constant [m^3/kg/s^2]
rhoG = rho*G;

X = X(:);
Y = Y(:);

Rx = [1 0 0;0 cosd(omegaX) sind(omegaX);0 -sind(omegaX) cosd(omegaX)];
Ry = [cosd(omegaY) 0 -sind(omegaY);0 1 0;sind(omegaY) 0 cosd(omegaY)];
Rz = [cosd(omegaZ) sind(omegaZ) 0;-sind(omegaZ) cosd(omegaZ) 0;0 0 1];
R = Rz*Ry*Rx;

Vstrike1 = [-R(2,1),R(1,1),0];
Vstrike1 = Vstrike1/norm(Vstrike1);
strike1 = atan2(Vstrike1(1),Vstrike1(2))*180/pi;
if isnan(strike1)
    strike1 = 0;
end
dip1 = acosd(R(3,1));

Vstrike2 = [-R(2,2),R(1,2),0];
Vstrike2 = Vstrike2/norm(Vstrike2);
strike2 = atan2(Vstrike2(1),Vstrike2(2))*180/pi;
if isnan(strike2)
    strike2 = 0;
end
dip2 = acosd(R(3,2));

Vstrike3 = [-R(2,3),R(1,3),0];
Vstrike3 = Vstrike3/norm(Vstrike3);
strike3 = atan2(Vstrike3(1),Vstrike3(2))*180/pi;
if isnan(strike3)
    strike3 = 0;
end
dip3 = acosd(R(3,3));

% Calculate contribution of the first PTD
if DVx~=0
    [ue1,un1,uv1,dgA1,dgMD1] = PTD_disp_grav_Surf(X,Y,X0,Y0,depth,...
        strike1,dip1,DVx,rhoG,nu);
else
    ue1 = zeros(size(X));
    un1 = zeros(size(X));
    uv1 = zeros(size(X));
    dgA1 = zeros(size(X));
    dgMD1 = zeros(size(X));
end

% Calculate contribution of the second PTD
if DVy~=0
    [ue2,un2,uv2,dgA2,dgMD2] = PTD_disp_grav_Surf(X,Y,X0,Y0,depth,...
        strike2,dip2,DVy,rhoG,nu);
else
    ue2 = zeros(size(X));
    un2 = zeros(size(X));
    uv2 = zeros(size(X));
    dgA2 = zeros(size(X));
    dgMD2 = zeros(size(X));
end

% Calculate contribution of the third PTD
if DVz~=0
    [ue3,un3,uv3,dgA3,dgMD3] = PTD_disp_grav_Surf(X,Y,X0,Y0,depth,...
        strike3,dip3,DVz,rhoG,nu);
else
    ue3 = zeros(size(X));
    un3 = zeros(size(X));
    uv3 = zeros(size(X));
    dgA3 = zeros(size(X));
    dgMD3 = zeros(size(X));
end

ue = ue1+ue2+ue3;
un = un1+un2+un3;
uv = uv1+uv2+uv3;

dgFA = gamma*uv; % Free-air effect

% using Newton's law for a point mass
dgDV = -rhoG*(DVx+DVy+DVz)*depth./((X-X0).^2+(Y-Y0).^2+depth.^2).^1.5;
dgDM = G*DM*depth./((X-X0).^2+(Y-Y0).^2+depth.^2).^1.5;

dgA = dgA1+dgA2+dgA3; % dgA=dgMD+dgSM!
dgMD = dgMD1+dgMD2+dgMD3;
dgSM = dgA-dgMD;
dg = dgFA+dgDV+dgDM+dgA;

function [ue,un,uv,dgA,dgMD]=PTD_disp_grav_Surf(X,Y,X0,Y0,depth,strike,...
    dip,DV,rhoG,nu)
% PTD_disp_grav_Surf calculates surface displacements and gravity changes 
% associated with a point tensile dislocation (PTD) in a uniform elastic 
% half-space (Okada 1985 and Okubo 1991).

% The reference coordinate system is EFCS (East, North, Up).

x = X(:)-X0;
y = Y(:)-Y0;

beta = strike-90; % due to Okada (1985) convention!
Rz = [cosd(beta) -sind(beta);sind(beta) cosd(beta)];
r_betaC = Rz*[x y]'; % transforming horizontal coordinates
x = r_betaC(1,:)';
y = r_betaC(2,:)';

d = depth;
r = (x.^2+y.^2+d.^2).^0.5;
q = y*sind(dip)-d*cosd(dip);

I1 = (1-2*nu)*y.*(1./r./(r+d).^2-x.^2.*(3*r+d)./r.^3./(r+d).^3);
I2 = (1-2*nu)*x.*(1./r./(r+d).^2-y.^2.*(3*r+d)./r.^3./(r+d).^3);
I3 = (1-2*nu)*x./r.^3-I2;
I5 = (1-2*nu)*(1./r./(r+d)-x.^2.*(2*r+d)./r.^3./(r+d).^2);

% Note: For a PTD M0 = DV*mu!
ue = DV/2/pi*(3*x.*q.^2./r.^5-I3*sind(dip)^2);
un = DV/2/pi*(3*y.*q.^2./r.^5-I1*sind(dip)^2);
uv = DV/2/pi*(3*d.*q.^2./r.^5-I5*sind(dip)^2);

r_betaD = Rz'*[ue un]'; % transforming horizontal displacements
ue = r_betaD(1,:)';
un = r_betaD(2,:)';


% calculate gravity changes (Okubo 1991)

x = -r_betaC(2,:)'; % due to a different convention for Rz in Okubo (1991)! 
% Note that y = r_betaC(1,:)', but it is not needed below. y contributes to
% the results only through r, which is invariant (independent of coordinate
% system conventions)!

% First: calculations for DV = 1, dip = 0 and dip = 90
dgA11 = 3*rhoG*d*x.^2./r.^5; % normal to x, tensile slip
dgA33 = 3*rhoG*d^3./r.^5; % normal to z, tensile slip
dgA13 = 3*rhoG*x*d^2./r.^5; % normal to z, shear slip along x

dgMD11 = rhoG*(1-2*nu)*(d./r.^3-1./r./(r+d)+x.^2.*(2*r+d)./r.^3./(r+d).^2);
dgMD33 = 0;
dgMD13 = 0;

% Second: calculations for generic DV and dip
dgA = dgCalc(dgA11,dgA33,dgA13,DV,dip);
dgMD = dgCalc(dgMD11,dgMD33,dgMD13,DV,dip);

function dg=dgCalc(dg11,dg33,dg13,DV,dip)
% dgCalc calculates the results for generic dip and potency values
dg = DV*(sind(dip)^2*dg11+sind(2*dip)*dg13+cosd(dip)^2*dg33);